/*
 *   Copyright (C) 2007 Matias Costa <m.costacano@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#include "group.h"
#include <QStringList>
#include "launcher.h"

void Group::add(const Launcher *e)
{
    double n = m_members.count();
    m_center.setX( (m_center.x()*n +e->pos().x() +e->sizeHint().width()/2.0)
                    /(n+1));
    m_center.setY( (m_center.y()*n +e->pos().y() +e->sizeHint().height()/2.0)
                    /(n+1));
    m_members.append(e);
    foreach (QString tag, e->tags().split(" ")) {
        ++m_tags[tag];
    }
}

void Group::remove(const Launcher *e) {
    m_members.removeAll(e);
    remove(e->pos().x()+e->sizeHint().width()/2.0, e->pos().y()+e->sizeHint().height()/2.0, e->tags());

}

void Group::remove(double x, double y, const QString tags)
{
    double n = m_members.count()+1;
    if ( (n-1) == 0 ) {
        return;
    }
    m_center.setX( (m_center.x()*n -x)
                    /(n-1));
    m_center.setY( (m_center.y()*n -y)
                    / (n-1));
    foreach (QString tag, tags.split(" ")) {
        --m_tags[tag];
        if (m_tags[tag] == 0) {
            m_tags.remove(tag);
        }
    }
}

void Group::join(Group &other)
{
    // These are double because anyway after will
    // be converted, so we assure the conversion
    // is done one time
    double n1 = count();
    double n2 = other.count();
    double n = n1 + n2;

    // recalculate center
    m_center.setX( (n1*m_center.x()) /n
            + (n2*other.m_center.x()) /n);
    m_center.setY( (n1*m_center.y()) /n
            + (n2*other.m_center.y()) /n);

    // add memebers
    m_members += other.m_members;

    // add tags
    foreach (const QString &tag, other.m_tags.keys()) {
        m_tags[tag] += other.m_tags[tag];
    }
}
