/*
 *   Copyright (C) 2007 Matias Costa <m.costacano@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#ifndef GROUP_H
#define GROUP_H
#include <QMap>
#include <QList>
#include <QString>
#include <QPointF>
#include <QStringList>

class Launcher;

/**
	@author M. Costa <m.costacano@gmail.com>
*/
class Group {
public:
    Group():
        m_center(0,0),
        m_valid(false)
    {};
        
    void add(const Launcher *e);
    void remove(const Launcher *e);
    void remove(double x, double y, const QString tags);

    /**
     * Adds all members of the group to
     * this group updating center and tag count.
     * Note: The group joined to this is empty after this call.
     *
     * @param other The group to be assimilated and cleaned
     */
    void join(Group &other);
    
    int count() const
    {
        return m_members.count();
    };
    
    bool contains(const Launcher *e) const
    {
        return m_members.contains(e);
    };
     
    const Launcher*  operator[](int i) const
    {
        return m_members[i];
    };

    QPointF center() const
    {
        return m_center;
    };

    bool isValid() const
    {
        return m_valid;
    };

    void setValid(bool valid)
    {
        m_valid = valid;
    };

    QStringList tags() const
    {
        return m_tags.keys();
    }

    const int tag(QString tag) const
    {
        return m_tags[tag];
    }

    const QList<const Launcher*>& members() const
    {
        return m_members;
    }
private:
    QPointF m_center;
    bool m_valid;
    QList<const Launcher*> m_members;
    QMap<QString,int> m_tags;
};

#endif
