/*
 *   Copyright (C) 2007 Matias Costa <m.costacano@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#ifndef SMARTGROUPS_H
#define SMARTGROUPS_H

#include <QObject>
#include <QList>
#include <QMap>
#include <QPointF>
#include <QDebug>

#include "group.h"
#include "launcher.h"

/**
    @author M. Costa <m.costacano@gmail.com>
*/

class StandardDesktop;

class SmartGroups: public QObject
{
    Q_OBJECT
public:
    SmartGroups(StandardDesktop *desktop);

    ~SmartGroups();

    void place(const QList<Launcher*>& items);

    void learn();

    // Groups
    int groupCount() const;
    int groupIndex(const Launcher *unit) const;
    const Group& group(const Launcher *unit) const;
    

    const Group& operator[](int index)
    {
        return m_groups[index];
    };
    
    void clear();
    void remove(const Launcher *e);
    void reset();

private:
    double euclideanDistance(const Launcher *a, const Launcher *b);
    
    double tagsDistance(const Group &c,
                        const QString &tags);
    double euclideanDistance(const QPointF &a, const QPointF &b);
    double groupDistance(const Group &c,
                         const Launcher *e, double Wt);
    
    int bestGroup(const Launcher *e);
    void divide(int c, const Launcher *u1, const Launcher *u2);
    bool validate(const Group &c) const;
    void balance();
    void join();

    QList<Group> m_groups;
    StandardDesktop *m_desktop;
};



#endif
