/*
 *   Copyright (C) 2007 Ivan Cukic <ivan.cukic+kde@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef UI_LANCELOTWINDOW_H_
#define UI_LANCELOTWINDOW_H_

#include <plasma/svg.h>
#include <KIcon>
#include <plasma/corona.h>
#include <plasma/widgets/icon.h>
//#include <plasma/widgets/vboxlayout.h>
//#include <plasma/widgets/hboxlayout.h>
#include <plasma/widgets/label.h>
#include "PositionLayout.h"

#include "ExtenderButton.h"
#include "ActionListView.h"
#include "Panel.h"

#include <kwindowsystem.h>
#include <QtCore>
#include <QtGui>

namespace Ui {
class LancelotWindow
{
    //Q_OBJECT
    
protected:
    // Plasma shell
    class CustomGraphicsView : public QGraphicsView {
    public:
        CustomGraphicsView  ( QWidget * parent = 0 )
            : QGraphicsView(parent), m_background(NULL) {};
        CustomGraphicsView  ( QGraphicsScene * scene, QWidget * parent = 0 )
            : QGraphicsView(scene, parent), m_background(NULL) {};
        void drawBackground (QPainter * painter, const QRectF & rect)
        {
            Q_UNUSED(rect);
            if (m_background) {
                m_background->resize(QSizeF(550, 500));
                m_background->paint(painter, 0, 0, "background");
            }
        };

    private:
        Plasma::Svg * m_background;
        friend class LancelotWindow;
    };
    
    Plasma::Svg * m_mainTheme;
    CustomGraphicsView * m_view;
    Plasma::Corona     * m_corona;
    QVBoxLayout        * m_layout;
    
    // Components
    Lancelot::PositionLayout * layoutMain;

    QList < Lancelot::ExtenderButton * > systemButtons;
    Lancelot::ExtenderButton * buttonSystemLockScreen;
    Lancelot::ExtenderButton * buttonSystemLogout;
    Lancelot::ExtenderButton * buttonSystemSwitchUser;
    
    QList < Lancelot::ExtenderButton * > categoryButtons;
    Lancelot::ExtenderButton * buttonCategoryApplications;
    Lancelot::ExtenderButton * buttonCategoryContacts;
    Lancelot::ExtenderButton * buttonCategoryDocuments;
    Lancelot::ExtenderButton * buttonCategorySystem;

    QList < Lancelot::ActionListView * > categoryLists;
    Lancelot::ActionListView * listCategoryApplications;
    Lancelot::ActionListView * listCategoryContacts;
    Lancelot::ActionListView * listCategoryDocuments;
    Lancelot::ActionListView * listCategorySystem;
    
    QList < Lancelot::Panel * > categoryPanels;
    Lancelot::Panel * panelCategoryApplications;
    Lancelot::Panel * panelCategoryContacts;
    Lancelot::Panel * panelCategoryDocuments;
    Lancelot::Panel * panelCategorySystem;
    
    void setupUi(QFrame * object) {
        // Init Plasma Shell
        
        object->setWindowFlags(Qt::FramelessWindowHint | Qt::Popup);
        KWindowSystem::setState( object->winId(), NET::SkipTaskbar | NET::SkipPager | NET::KeepAbove );
        
        m_corona = new Plasma::Corona(object);
        m_layout = new QVBoxLayout(object);
        m_layout->setContentsMargins (0, 0, 0, 0);
        object->setLayout(m_layout);
        
        m_view = new CustomGraphicsView(m_corona, object);
        m_view->setFrameStyle(QFrame::NoFrame);
        m_view->setCacheMode(QGraphicsView::CacheBackground);
        m_view->setInteractive(true);
        m_view->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff); // TODO: Change to off
        m_view->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);   // TODO: Change to off

        m_view->m_background = new Plasma::Svg("lancelot/theme");
        m_view->m_background->setContentType(Plasma::Svg::ImageSet);
        
        m_layout->addWidget(m_view);
        
        // Init layouts
        layoutMain = new Lancelot::PositionLayout();
        
        // System buttons
        
        systemButtons.append(buttonSystemLockScreen = 
            new Lancelot::ExtenderButton(new KIcon("system-lock-screen"), "Lock Session", ""));
        systemButtons.append(buttonSystemLogout = 
            new Lancelot::ExtenderButton(new KIcon("system-log-out"), "Log Out...", ""));
        systemButtons.append(buttonSystemSwitchUser = 
            new Lancelot::ExtenderButton(new KIcon("switchuser"), "Switch User", ""));
        
        categoryButtons.append(buttonCategoryApplications = 
            new Lancelot::ExtenderButton(new KIcon("make-kdevelop"), "Applications", ""));
        categoryButtons.append(buttonCategoryContacts = 
            new Lancelot::ExtenderButton(new KIcon("kontact"), "Contacts", ""));
        categoryButtons.append(buttonCategoryDocuments = 
            new Lancelot::ExtenderButton(new KIcon("applications-office"), "Documents", ""));
        categoryButtons.append(buttonCategorySystem = 
            new Lancelot::ExtenderButton(new KIcon("video-display"), "System", ""));

        float dist = 136;
        foreach (Lancelot::ExtenderButton * button, systemButtons) {
            button->setActivationMethod(Lancelot::ExtenderButton::EXTENDER);
            button->setExtenderPosition(Lancelot::ExtenderButton::BOTTOM);
            button->setIconSize(QSize(24, 24));
            button->setZValue(1);

            m_corona->addItem(button);
            layoutMain->addItem(button, Lancelot::PositionLayout::Position(dist, -46, dist + 132, -20));
            dist += 136;
        }
 
        dist = -56;
        foreach (Lancelot::ExtenderButton * button, categoryButtons) {
            button->setActivationMethod(Lancelot::ExtenderButton::HOVER);
            button->setIconSize(QSize(48, 48)); // QSize(48, 48));
            button->setZValue(1);
            button->setInnerOrientation(Lancelot::BaseWidget::VERTICAL);

            m_corona->addItem(button);
            layoutMain->addItem(button, Lancelot::PositionLayout::Position(0, dist - 100, 136, dist));
            dist -= 104;
        }

        categoryLists.append(listCategoryApplications = 
            new Lancelot::ActionListView());
        categoryLists.append(listCategoryContacts =
            new Lancelot::ActionListView());
        categoryLists.append(listCategoryDocuments =
            new Lancelot::ActionListView());
        categoryLists.append(listCategorySystem =
            new Lancelot::ActionListView());
        
        categoryPanels.append(panelCategoryApplications = 
            new Lancelot::Panel(new KIcon("about-kde"), "Applications"));
        categoryPanels.append(panelCategoryContacts =
            new Lancelot::Panel("Category"));
        categoryPanels.append(panelCategoryDocuments =
            new Lancelot::Panel("Documents"));
        categoryPanels.append(panelCategorySystem =
            new Lancelot::Panel());
        
        int i = 0;
        
        foreach (Lancelot::Panel * panel, categoryPanels) {
            m_corona->addItem(panel);
            layoutMain->addItem(panel, Lancelot::PositionLayout::Position(130 + 40 * i, 32, 150 + 170 + 40 * i, -56));
            
            panel->setWidget(categoryLists.at(i++));
        }
        
        // TODO: Make it show exactly what you want!!! It works?
        object->resize(550, 500);
        layoutMain->setGeometry(QRectF(0, 0, 550, 500));
        //m_view->ensureVisible(QRectF(0, 0, 550, 500));
        m_corona->setSceneRect(QRectF(0, 0, 550, 500));

    }
};

} // namespace Ui

#endif /*UI_LANCELOTWINDOW_H_*/
