/*
 *   Copyright (C) 2007 Petri Damsten <damu@iki.fi>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "executable.h"
#include <KDebug>

EEProcess::EEProcess(QObject* parent)
    : QProcess(parent), m_outIndex(0), m_errIndex(0)
{
}

EEProcess::~EEProcess()
{
}

void EEProcess::run(const QString &executable)
{
  connect(this, SIGNAL(readyReadStandardOutput()), this, SLOT(stdout()));
  connect(this, SIGNAL(readyReadStandardError()), this, SLOT(stderr()));
  connect(this, SIGNAL(finished(int, QProcess::ExitStatus)),
          this, SLOT(exit(int, QProcess::ExitStatus)));
  m_executable = executable;
  setProcessChannelMode(QProcess::SeparateChannels);
  start(m_executable, QIODevice::ReadOnly);
}

void EEProcess::stderr()
{
    int found;
    QRegExp re("[\n\r]");
    QByteArray ba = readAllStandardError();

    m_err += QString::fromLocal8Bit(ba.data(), ba.count());
    kDebug() << m_err;
    while ((found = m_err.indexOf(re, m_errIndex)) != -1) {
        m_errList.append(m_err.mid(m_errIndex, found - m_errIndex));
        m_errIndex = found + 1;
    }
}

void EEProcess::stdout()
{
    int found;
    QRegExp re("[\n\r]");
    QByteArray ba = readAllStandardOutput();

    m_out += QString::fromLocal8Bit(ba.data(), ba.count());
    kDebug() << m_out;
    while ((found = m_out.indexOf(re, m_outIndex)) != -1) {
        m_outList.append(m_out.mid(m_outIndex, found - m_outIndex));
        m_outIndex = found + 1;
    }
}

void EEProcess::exit(int exitCode, QProcess::ExitStatus exitStatus)
{
  Q_UNUSED(exitCode)
  Q_UNUSED(exitStatus)

  if (m_outIndex < m_out.length()) {
    m_outList.append(m_out.mid(m_outIndex));
  }
  if (m_errIndex < m_err.length()) {
    m_errList.append(m_err.mid(m_errIndex));
  }
  emit ready(this);
}

ExecutableEngine::ExecutableEngine(QObject* parent, const QVariantList& args)
    : Plasma::DataEngine(parent)
{
    Q_UNUSED(args)
    setMinimumUpdateInterval(1000);
}

ExecutableEngine::~ExecutableEngine()
{
}

bool ExecutableEngine::updateSource(const QString &name)
{
    EEProcess *process = new EEProcess();
    connect(process, SIGNAL(ready(EEProcess *)),
            this, SLOT(sourceUpdated(EEProcess *)));
    process->run(name);
    return true;
}

bool ExecutableEngine::sourceRequested(const QString &name)
{
    setData(name, DataEngine::Data());
    updateSource(name);
    return true;
}

void ExecutableEngine::sourceUpdated(EEProcess *process)
{
    QString name = process->executable();
    setData(name, "exit code", process->exitCode());
    setData(name, "exit status", process->exitStatus());
    setData(name, "stdout", process->out());
    setData(name, "stderr", process->err());
    process->deleteLater();
}

#include "executable.moc"
