/*
 *   Copyright (C) 2007 Petri Damsten <damu@iki.fi>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef EXECUTABLE_DATAENGINE_H
#define EXECUTABLE_DATAENGINE_H

#include <QProcess>
#include "plasma/dataengine.h"

class EEProcess;

class ExecutableEngine : public Plasma::DataEngine
{
    Q_OBJECT

    public:
        ExecutableEngine(QObject *parent, const QVariantList &args);
        ~ExecutableEngine();

    protected:
        bool sourceRequested(const QString &name);
        bool updateSource(const QString &name);

    public slots:
        void sourceUpdated(EEProcess *process);
};

class EEProcess : public QProcess
{
    Q_OBJECT

    public:
        EEProcess(QObject* parent = 0);
        ~EEProcess();

        QString executable() { return m_executable; };
        QStringList out() { return m_outList; };
        QStringList err() { return m_errList; };

        void run(const QString &executable);

    signals:
        void ready(EEProcess *);

    private slots:
        void stdout();
        void stderr();
        void exit(int exitCode, QProcess::ExitStatus exitStatus);

    private:
        QString m_out;
        QString m_err;
        QStringList m_outList;
        QStringList m_errList;
        int m_outIndex;
        int m_errIndex;
        QString m_executable;
};

K_EXPORT_PLASMA_DATAENGINE(executable, ExecutableEngine)

#endif
