/*  Plasma List Widget
    Copyright (C) 2007  Tim Beaulen <tbscope@gmail.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef PLASMA_LISTWIDGET_H
#define PLASMA_LISTWIDGET_H

#include <plasma/widgets/widget.h>

/* ---[ TODO List ]---

   - Implement an empty skeleton widget
   - Draw a frame
   - Create a list item class
   - List item management, the standard stuff from QListWidget.
   - Use the paint function of the list item class to draw the items (using layouts)
   - Implement scrolling.
   - Events (mouse click, double click, keypress, ...)
   - Documentation

   ---[ Technical Ideas ]---

   - Make it behave like a QListWidget
   - Don't forget to style the widget according to the current style(sheet).
   - No headers are needed, it's single column. A treewidget has multiple columns.
   - It's not just a matter of drawing a complex control or primitive.
     First draw a frame, the all the items. Each item consisting out of an icon (if available) and a label (without frame).
     Draw scrollbars.
   - Only draw those items that can be (partially) seen.
*/

namespace Plasma
{

class PLASMA_EXPORT ListWidget : public Plasma::Widget
{
    Q_OBJECT

    public:
        ListWidget(Widget *parent);
        virtual ~ListWidget();

        Qt::Orientations expandingDirectories() const;

        //bool hasHeightForWidth() const; // No height for width (at least not for the list widget) 
        //bool hasWidthForHeight() const; // Idem

        QSizeF sizeHint() const;

        QRectF boundingRect() const;

        void paintWidget(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
};

}

#endif // PLASMA_LISTWIDGET_H

